unit BWStack;

interface

uses Classes, SysUtils;

type EBWStackError = class (exception)
     end;

type TBWStack = class (TObject)
      private
       FList : PPointerList;
       FCount: Integer;
       FCapacity: Integer;
       function Get(index:integer):pointer;
       procedure Put(index:integer; value:pointer);
       procedure Grow;
       procedure SetCapacity(NewCapacity: Integer);
      public
       destructor Destroy; override;
       procedure Clear;
       procedure Push(value:pointer);
       function Pop:pointer;
       property Capacity:integer read FCapacity write SetCapacity;
       property Items[Index: Integer]: Pointer read Get write Put; default;
       property Count : integer read FCount;
     end;

implementation

function TBWStack.Get(index:integer):pointer;
begin
  result:=FList^[index];
end;

procedure TBWStack.Put(index:integer; value:pointer);
begin
  FList^[index]:=value;
end;

destructor TBWStack.Destroy;
begin
  FreeMem(FList,FCapacity);
  inherited Destroy;
end;

procedure TBWStack.Grow;
var
  Delta: Integer;
begin
  if FCapacity > 64 then Delta := FCapacity div 4 else
    if FCapacity > 8 then Delta := 16 else
      Delta := 4;
  SetCapacity(FCapacity + Delta);
end;

procedure TBWStack.SetCapacity(NewCapacity: Integer);
begin
  if (NewCapacity < FCount) or (NewCapacity > MaxListSize) then
    raise EBWStackError.Create('Invalid BWStack capacity.');
  if NewCapacity <> FCapacity then
  begin
    ReallocMem(FList, NewCapacity * SizeOf(Pointer));
    FCapacity := NewCapacity;
  end;
end;

procedure TBWStack.Push;
begin
  if FCount = FCapacity then Grow;
  FList^[FCount] := value;
  Inc(FCount);
end;

function TBWStack.Pop;
begin
  if FCount=0 then raise EBWStackError.Create('BWStack is empty.');
  result:=FList^[FCount-1];
  dec(FCount);
end;

procedure TBWStack.Clear;
begin
  FCount:=0;
  SetCapacity(0);
end;

end.
